@@ -21,16 +21,16 @@
  * specific prior written permission.
  * 
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
- * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
- * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
- * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
- * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
- * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
- * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
- * POSSIBILITY OF SUCH DAMAGE.
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+ * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 /**
@@ -41,9 +41,11 @@
 
 #include "config.h"
 #include <ctype.h>
-#include <ldns/ldns.h>
+#include <stdarg.h>
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #include "util/log.h"
-
 #include "util/configyyrename.h"
 #include "util/config_file.h"
 #include "util/configparser.h"
@@ -53,6 +55,8 @@
 #include "util/regional.h"
 #include "util/fptr_wlist.h"
 #include "util/data/dname.h"
+#include "ldns/wire2str.h"
+#include "ldns/parseutil.h"
 #ifdef HAVE_GLOB_H
 # include <glob.h>
 #endif
@@ -134,6 +138,7 @@ config_create(void)
 	cfg->prefetch_key = 0;
 	cfg->infra_cache_slabs = 4;
 	cfg->infra_cache_numhosts = 10000;
+	cfg->delay_close = 0;
 	if(!(cfg->outgoing_avail_ports = (int*)calloc(65536, sizeof(int))))
 		goto error_exit;
 	init_outgoing_availports(cfg->outgoing_avail_ports, 65536);
@@ -152,6 +157,7 @@ config_create(void)
 	cfg->if_automatic = 0;
 	cfg->so_rcvbuf = 0;
 	cfg->so_sndbuf = 0;
+	cfg->so_reuseport = 0;
 	cfg->num_ifs = 0;
 	cfg->ifs = NULL;
 	cfg->num_out_ifs = 0;
@@ -361,6 +367,7 @@ int config_set_option(struct config_file* cfg, const char* opt,
 	else S_SIZET_OR_ZERO("jostle-timeout:", jostle_time)
 	else S_MEMSIZE("so-rcvbuf:", so_rcvbuf)
 	else S_MEMSIZE("so-sndbuf:", so_sndbuf)
+	else S_YNO("so-reuseport:", so_reuseport)
 	else S_MEMSIZE("rrset-cache-size:", rrset_cache_size)
 	else S_POW2("rrset-cache-slabs:", rrset_cache_slabs)
 	else S_YNO("prefetch:", prefetch)
@@ -372,6 +379,7 @@ int config_set_option(struct config_file* cfg, const char* opt,
 	else S_NUMBER_OR_ZERO("infra-host-ttl:", host_ttl)
 	else S_POW2("infra-cache-slabs:", infra_cache_slabs)
 	else S_SIZET_NONZERO("infra-cache-numhosts:", infra_cache_numhosts)
+	else S_NUMBER_OR_ZERO("delay-close:", delay_close)
 	else S_STR("chroot:", chrootdir)
 	else S_STR("username:", username)
 	else S_STR("directory:", directory)
@@ -606,6 +614,7 @@ config_get_option(struct config_file* cfg, const char* opt,
 	else O_UNS(opt, "jostle-timeout", jostle_time)
 	else O_MEM(opt, "so-rcvbuf", so_rcvbuf)
 	else O_MEM(opt, "so-sndbuf", so_sndbuf)
+	else O_YNO(opt, "so-reuseport", so_reuseport)
 	else O_MEM(opt, "rrset-cache-size", rrset_cache_size)
 	else O_DEC(opt, "rrset-cache-slabs", rrset_cache_slabs)
 	else O_YNO(opt, "prefetch-key", prefetch_key)
@@ -615,6 +624,7 @@ config_get_option(struct config_file* cfg, const char* opt,
 	else O_DEC(opt, "infra-host-ttl", host_ttl)
 	else O_DEC(opt, "infra-cache-slabs", infra_cache_slabs)
 	else O_MEM(opt, "infra-cache-numhosts", infra_cache_numhosts)
+	else O_UNS(opt, "delay-close", delay_close)
 	else O_YNO(opt, "do-ip4", do_ip4)
 	else O_YNO(opt, "do-ip6", do_ip6)
 	else O_YNO(opt, "do-udp", do_udp)
@@ -1085,7 +1095,7 @@ cfg_convert_timeval(const char* str)
 	if (tm.tm_min < 0 || tm.tm_min > 59)	return 0;
 	if (tm.tm_sec < 0 || tm.tm_sec > 59)	return 0;
 	/* call ldns conversion function */
-	t = mktime_from_utc(&tm);
+	t = sldns_mktime_from_utc(&tm);
 	return t;
 }
 
@@ -1233,9 +1243,9 @@ strlen_after_chroot(const char* fname, struct config_file* cfg, int use_chdir)
 char*
 fname_after_chroot(const char* fname, struct config_file* cfg, int use_chdir)
 {
-	size_t len = strlen_after_chroot(fname, cfg, use_chdir);
+	size_t len = strlen_after_chroot(fname, cfg, use_chdir)+1;
 	int slashit = 0;
-	char* buf = (char*)malloc(len+1);
+	char* buf = (char*)malloc(len);
 	if(!buf)
 		return NULL;
 	buf[0] = 0;
@@ -1243,14 +1253,14 @@ fname_after_chroot(const char* fname, struct config_file* cfg, int use_chdir)
 	if(cfg->chrootdir && cfg->chrootdir[0] && 
 		strncmp(cfg->chrootdir, fname, strlen(cfg->chrootdir)) == 0) {
 		/* already full pathname, return it */
-		strncpy(buf, fname, len);
-		buf[len] = 0;
+		(void)strlcpy(buf, fname, len);
+		buf[len-1] = 0;
 		return buf;
 	}
 	/* chroot */
 	if(cfg->chrootdir && cfg->chrootdir[0]) {
 		/* start with chrootdir */
-		strncpy(buf, cfg->chrootdir, len);
+		(void)strlcpy(buf, cfg->chrootdir, len);
 		slashit = 1;
 	}
 #ifdef UB_ON_WINDOWS
@@ -1264,21 +1274,21 @@ fname_after_chroot(const char* fname, struct config_file* cfg, int use_chdir)
 	} else if(cfg->directory && cfg->directory[0]) {
 		/* prepend chdir */
 		if(slashit && cfg->directory[0] != '/')
-			strncat(buf, "/", len-strlen(buf));
+			(void)strlcat(buf, "/", len);
 		/* is the directory already in the chroot? */
 		if(cfg->chrootdir && cfg->chrootdir[0] && 
 			strncmp(cfg->chrootdir, cfg->directory, 
 			strlen(cfg->chrootdir)) == 0)
-			strncat(buf, cfg->directory+strlen(cfg->chrootdir), 
-				   len-strlen(buf));
-		else strncat(buf, cfg->directory, len-strlen(buf));
+			(void)strlcat(buf, cfg->directory+strlen(cfg->chrootdir), 
+				   len);
+		else (void)strlcat(buf, cfg->directory, len);
 		slashit = 1;
 	}
 	/* fname */
 	if(slashit && fname[0] != '/')
-		strncat(buf, "/", len-strlen(buf));
-	strncat(buf, fname, len-strlen(buf));
-	buf[len] = 0;
+		(void)strlcat(buf, "/", len);
+	(void)strlcat(buf, fname, len);
+	buf[len-1] = 0;
 	return buf;
 }
 
@@ -1329,7 +1339,7 @@ cfg_parse_local_zone(struct config_file* cfg, const char* val)
 		log_err("syntax error: bad zone name: %s", val);
 		return 0;
 	}
-	strncpy(buf, name, (size_t)(name_end-name));
+	(void)strlcpy(buf, name, sizeof(buf));
 	buf[name_end-name] = '\0';
 
 	type = last_space_pos(name_end);
@@ -1519,18 +1529,11 @@ char* errinf_to_str(struct module_qstate* qstate)
 	size_t left = sizeof(buf);
 	struct config_strlist* s;
 	char dname[LDNS_MAX_DOMAINLEN+1];
-	char* t = ldns_rr_type2str(qstate->qinfo.qtype);
-	char* c = ldns_rr_class2str(qstate->qinfo.qclass);
-	if(!t || !c) {
-		free(t);
-		free(c);
-		log_err("malloc failure in errinf_to_str");
-		return NULL;
-	}
+	char t[16], c[16];
+	sldns_wire2str_type_buf(qstate->qinfo.qtype, t, sizeof(t));
+	sldns_wire2str_class_buf(qstate->qinfo.qclass, c, sizeof(c));
 	dname_str(qstate->qinfo.qname, dname);
 	snprintf(p, left, "validation failure <%s %s %s>:", dname, t, c);
-	free(t);
-	free(c);
 	left -= strlen(p); p += strlen(p);
 	if(!qstate->errinf)
 		snprintf(p, left, " misc failure");
@@ -1548,21 +1551,13 @@ void errinf_rrset(struct module_qstate* qstate, struct ub_packed_rrset_key *rr)
 {
 	char buf[1024];
 	char dname[LDNS_MAX_DOMAINLEN+1];
-	char *t, *c;
+	char t[16], c[16];
 	if(qstate->env->cfg->val_log_level < 2 || !rr)
 		return;
-	t = ldns_rr_type2str(ntohs(rr->rk.type));
-	c = ldns_rr_class2str(ntohs(rr->rk.rrset_class));
-	if(!t || !c) {
-		free(t);
-		free(c);
-		log_err("malloc failure in errinf_rrset");
-		return;
-	}
+	sldns_wire2str_type_buf(ntohs(rr->rk.type), t, sizeof(t));
+	sldns_wire2str_class_buf(ntohs(rr->rk.rrset_class), c, sizeof(c));
 	dname_str(rr->rk.dname, dname);
 	snprintf(buf, sizeof(buf), "for <%s %s %s>", dname, t, c);
-	free(t);
-	free(c);
 	errinf(qstate, buf);
 }
 
