@@ -161,19 +161,21 @@ extern int yydebug;
      VAR_PREFETCH = 370,
      VAR_PREFETCH_KEY = 371,
      VAR_SO_SNDBUF = 372,
-     VAR_HARDEN_BELOW_NXDOMAIN = 373,
-     VAR_IGNORE_CD_FLAG = 374,
-     VAR_LOG_QUERIES = 375,
-     VAR_TCP_UPSTREAM = 376,
-     VAR_SSL_UPSTREAM = 377,
-     VAR_SSL_SERVICE_KEY = 378,
-     VAR_SSL_SERVICE_PEM = 379,
-     VAR_SSL_PORT = 380,
-     VAR_FORWARD_FIRST = 381,
-     VAR_STUB_FIRST = 382,
-     VAR_MINIMAL_RESPONSES = 383,
-     VAR_RRSET_ROUNDROBIN = 384,
-     VAR_MAX_UDP_SIZE = 385
+     VAR_SO_REUSEPORT = 373,
+     VAR_HARDEN_BELOW_NXDOMAIN = 374,
+     VAR_IGNORE_CD_FLAG = 375,
+     VAR_LOG_QUERIES = 376,
+     VAR_TCP_UPSTREAM = 377,
+     VAR_SSL_UPSTREAM = 378,
+     VAR_SSL_SERVICE_KEY = 379,
+     VAR_SSL_SERVICE_PEM = 380,
+     VAR_SSL_PORT = 381,
+     VAR_FORWARD_FIRST = 382,
+     VAR_STUB_FIRST = 383,
+     VAR_MINIMAL_RESPONSES = 384,
+     VAR_RRSET_ROUNDROBIN = 385,
+     VAR_MAX_UDP_SIZE = 386,
+     VAR_DELAY_CLOSE = 387
    };
 #endif
 /* Tokens.  */
@@ -292,19 +294,21 @@ extern int yydebug;
 #define VAR_PREFETCH 370
 #define VAR_PREFETCH_KEY 371
 #define VAR_SO_SNDBUF 372
-#define VAR_HARDEN_BELOW_NXDOMAIN 373
-#define VAR_IGNORE_CD_FLAG 374
-#define VAR_LOG_QUERIES 375
-#define VAR_TCP_UPSTREAM 376
-#define VAR_SSL_UPSTREAM 377
-#define VAR_SSL_SERVICE_KEY 378
-#define VAR_SSL_SERVICE_PEM 379
-#define VAR_SSL_PORT 380
-#define VAR_FORWARD_FIRST 381
-#define VAR_STUB_FIRST 382
-#define VAR_MINIMAL_RESPONSES 383
-#define VAR_RRSET_ROUNDROBIN 384
-#define VAR_MAX_UDP_SIZE 385
+#define VAR_SO_REUSEPORT 373
+#define VAR_HARDEN_BELOW_NXDOMAIN 374
+#define VAR_IGNORE_CD_FLAG 375
+#define VAR_LOG_QUERIES 376
+#define VAR_TCP_UPSTREAM 377
+#define VAR_SSL_UPSTREAM 378
+#define VAR_SSL_SERVICE_KEY 379
+#define VAR_SSL_SERVICE_PEM 380
+#define VAR_SSL_PORT 381
+#define VAR_FORWARD_FIRST 382
+#define VAR_STUB_FIRST 383
+#define VAR_MINIMAL_RESPONSES 384
+#define VAR_RRSET_ROUNDROBIN 385
+#define VAR_MAX_UDP_SIZE 386
+#define VAR_DELAY_CLOSE 387
 
 
 
@@ -318,7 +322,7 @@ typedef union YYSTYPE
 
 
 /* Line 2049 of yacc.c  */
-#line 322 "util/configparser.h"
+#line 326 "util/configparser.h"
 } YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
